if not DotIt then
	error("DotIt has to be loaded first")
end
local newList, del, deepDel, deepCopy = DotIt.sb:GetRecyclingFunctions()
local cooldowns = DotIt:NewModule("cooldowns")

local oncd = newList()
local mincd = 3
local match = string.match
local GetSpellLink = GetSpellLink
local GetSpellName = GetSpellName
local GetSpellCooldown = GetSpellCooldown

local books = {
	"BOOKTYPE_SPELL",
	"BOOKTYPE_PET"
}
local function getSpellId(link)
	return match(link, "Hspell:(%d+)")
end
local function getItemId(link)
	return match(link, "Hitem:(%d+)")
end
local function spellNeeded(id)
	return DotIt:GetSpellInfo("Cooldowns", id)
end
local function itemNeeded(id)
	return DotIt:GetSpellInfo("ItemCooldowns", id)
end

local start, duration, enabled, id, dur
local function spellUpdate()
	for _, book in pairs(books) do
		local i = 1
		local link = GetSpellLink(i, book)
		local name = GetSpellName(i, book)
		
		local prvname = ""
		while link do
			
			if prvname ~= name then
				start, duration, enabled = GetSpellCooldown(i, book)
				-- for example vanish uses this mechanic
				if enabled == 0 then
					if oncd[link] then
						id = oncd[link]
						oncd[link] = nil
						DotIt:SpellCooldownEnd(id)
					end
				else
					-- this spell is ready
					if start == 0 then
						if oncd[link] then
							id = oncd[link]
							oncd[link] = nil
							DotIt:SpellCooldownEnd(id)
						end
					else
						id = getSpellId(link)
						if spellNeeded(id) then
							dur = duration - (GetTime() - start)
							if dur > mincd then
								oncd[link] = id
								DotIt:SpellCooldownStart(id, dur)
							end
						end
					end
				end
			end
			
			i = i + 1
			prvname = name
			name = GetSpellName(i, book)
			link = GetSpellLink(i, book)
		end
	end
end
local function bagUpdate()
	for i = 0, 4 do 
		for j = 1, GetContainerNumSlots(i) do
			local start, duration, enable = GetContainerItemCooldown(i, j)
			if start ~= 0 then
				local link = GetContainerItemLink(i, j)
				local id = getItemId(link)
				if itemNeeded(id) then
					local dur = duration - (GetTime() - start)
					DotIt:StartItemCooldown(id, dur)
				end
			end
		end
	end
end
local function inventoryUpdate()
	for i = 1, 23 do 
		local start, duration, enable = GetInventoryItemCooldown("player", i)
		if start ~= 0 then
			local link = GetInventoryItemLink("player", i)
			local id = getItemId(link)
			if itemNeeded(id) then
				local dur = duration - (GetTime() - start)
				DotIt:StartItemCooldown(id, dur)
			end
		end
	end
end

function DotIt:SpellUpdate()
	spellUpdate()
	cooldowns:Init()
end
function cooldowns:Init()
	-- spells
	DotIt:RegisterEvent("SPELL_UPDATE_COOLDOWN", function() spellUpdate() end)
	DotIt:RegisterEvent("SPELLS_CHANGED", function() spellUpdate() end)
	DotIt:RegisterEvent("PET_BAR_UPDATE_COOLDOWN", function() spellUpdate() end)
	DotIt:RegisterEvent("ZONE_CHANGED_NEW_AREA", function() spellUpdate() end)
	
	-- items (do not need to call)
	DotIt:RegisterEvent("BAG_UPDATE_COOLDOWN", function() bagUpdate() end)
	DotIt:RegisterEvent("UNIT_INVENTORY_CHANGED", function() inventoryUpdate() end)
	DotIt:RegisterEvent("ZONE_CHANGED_NEW_AREA", function() bagUpdate() inventoryUpdate() end)
end